#ifndef OBJECT_H
#define OBJECT_H

#include "sprite.h"
#include "globals.h"
#include "features.h"

template <class T>class Object
{
	public:
		Object();
		Object(const char *);
		~Object();
		void init(const char *);
		int getw();
		int geth();
		void clear();
		void draw();
		void move();
		void set_range(const int, const int, const int, const int);
		void reset();

	protected:

		struct object_struct
		{
			T data;
			Sprite sprite;
			int speedy, speedx;
			object_struct *next;
		};

		object_struct *add_object(const int, const int, const int, const int);
		object_struct *object_collision(Sprite &);
		object_struct *next_object(object_struct *);
		bool del_object(object_struct *);

		BSprite bsprite;
		bool stop_on_pause;

	private:
		object_struct *head_object;
		int maxx, minx, miny, maxy;
};

template <class T>Object<T>::Object() : stop_on_pause(false), head_object(NULL)
{
}

template <class T>Object<T>::Object(const char *path) : stop_on_pause(false), head_object(NULL)
{
	init(path);
}

template <class T>Object<T>::~Object()
{
	while(del_object(head_object));
}

template <class T>void Object<T>::clear()
{
	while(del_object(head_object));
}

template <class T>void Object<T>::init(const char *path)
{
	bsprite.init(path);
	set_range(0 - bsprite.getw(), 640, 70 - bsprite.geth(), 470);
}

template <class T>void Object<T>::reset()
{
	bsprite.reset();
}

template <class T>void Object<T>::set_range(const int a, const int b, const int c, const int d)
{
	minx = a;
	maxx = b;
	miny = c;
	maxy = d;
}

template <class T>struct Object<T>::object_struct *Object<T>::object_collision(Sprite &a)
{
	object_struct *object;
	for(object = head_object; object != NULL; object = object->next)
	{
		if(is_collision(a, object->sprite))
		{
			return object;
		}
	}
	return NULL;
}

template <class T>bool Object<T>::del_object(object_struct *object)
{
	object_struct *i;
	if(object == NULL || head_object == NULL)
	{
		return false;
	}
	if(head_object == object)
	{
		head_object = object->next;
		delete object;
		return true;
	}
	for(i = head_object; i != NULL && i->next != object; i = i->next);
	i->next = object->next;
	delete object;
	return true;
}

template <class T>void Object<T>::draw()
{
	object_struct *object;
	for(object = head_object; object != NULL; object = object->next)
	{
		if(!object->sprite.is_visible())
		{
			del_object(object);
		}
		if(stop_on_pause)
		{
			if(game_paused)
			{
				object->sprite.stop_anim();
			}
			else
			{
				object->sprite.start_anim();
			}
		}
		object->sprite.draw();
	}
}

template <class T>void Object<T>::move()
{
	object_struct *object, *next;
	for(object = head_object; object != NULL; object = next)
	{
		next = object->next;
		object->sprite.addx(-(float)object->speedx * dt * 0.5);
		object->sprite.addy(-(float)object->speedy * dt * 0.5);
		if(!object->sprite.is_visible() || object->sprite.getx() < minx || object->sprite.getx() > maxx || object->sprite.gety() < miny || object->sprite.gety() > maxy)
		{
			del_object(object);
			continue;
		}
	}
}

template <class T>int Object<T>::geth()
{
	return bsprite.geth();
}

template <class T>int Object<T>::getw()
{
	return bsprite.getw();
}

template <class T>struct Object<T>::object_struct *Object<T>::next_object(object_struct *object)
{
	if(object == NULL)
	{
		return head_object;
	}
	return object->next;
}

template <class T>struct Object<T>::object_struct *Object<T>::add_object(const int x, const int y, const int speedx = 0, const int speedy = 0)
{
	object_struct *object;

	if(!bsprite.is_init_done())
	{
		return NULL;
	}
	object = new object_struct;
	object->sprite.init(bsprite);
	object->speedx = speedx;
	object->speedy = speedy;

	object->sprite.setxy(x, y);
	object->next = head_object;
	head_object = object;
	return object; 
}

#endif


