#include "spacerider.h"
#include "globals.h"

#include <iostream>
#include <exception>
#include <string>

using namespace std;

extern "C"
{
#include <SDL_ttf.h>
#include <SDL_mixer.h>
#include <SDL_net.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
}


Spacerider::Spacerider(const int argc, char *argv[]) : fullscreen(FULLSCREEN), game_over(false), online_highscore(ONLINE_HIGHSCORE), score_sent(false), name_input(true), highscore_thread_started(false), scroll(0), last_enemy(0), score(0), lives(LIVES), fps(FPS) 
{
	Font loading;
	int optchr, i;
	char tmp[256], *n;
	struct stat file_info;

	if((n = getenv("USER")) != NULL)
	{
		/* set default user-name to username */
		strncpy(name, n, 16);	
	}
	else
	{
		/* fill name with '\0's */
		memset(name, '\0', 16);
	}
	
	/* initialize global variables */
	screen = NULL;
	gt = 0;
	game_paused = false;
	strncpy(data_prefix, DEFAULT_DATA_PREFIX, 256);
	
	/* try to autodetect the data-prefix */
	if((stat("data", &file_info) == 0 && S_ISDIR(file_info.st_mode)) && (stat(data_prefix, &file_info) != 0 || !S_ISDIR(file_info.st_mode)))
	{
		data_prefix[0] = '\0';
	}
	
	/* parse arguments */
	while((optchr = getopt(argc, argv, "hvwfs:pl:den:t:")) != EOF)
	{
		switch(optchr)
		{
			case 'h':
				/* print out a small parameter list */
				cout << endl;
				cout << "Help:" << endl;
				cout << "-h         parameter list" << endl;
				cout << "-v         version information" << endl;
				cout << "-w         switch to window-mode (can be very slow)" << endl;
				cout << "-f         switch to fullscreen-mode" << endl;
				cout << "-p         start game paused" << endl;
				cout << "-s FPS     specify the max. fps" << endl;
				cout << "-l VOLUME  specify the sound-volume (0 - " << SDL_MIX_MAXVOLUME << ")" << endl;
				cout << "-n NAME    use this name for the online-highscore" << endl;
				cout << "-d         disable online-highscore" << endl;
				cout << "-e         enable online-highscore" << endl;
				cout << "-t PATH    change the data-prefix path" << endl;
				throw "";
				break;
			case 'v':
				/* show version, build-date, author, homepage ... */
				cout << endl;
				cout << "spacerider " << VERSION << ", built on " << __DATE__ << "." << endl;
				cout << "(c) Dominik Hackl [dominik@hackl.dhs.org]" << endl;
				cout << "Homepage: http://www.hackl.dhs.org/spacerider" << endl;
				cout << "ALL PARTS OF SPACERIDER ARE UNDER THE TERMS OF GPL. SEE DOCUMENTATION FOR MORE INFO." << endl;
				throw "";
				break;
			case 'f':
				/* switch to fullscreen-mode */
				fullscreen = true;
				break;
			case 'w':
				/* switch to window-mode */
				fullscreen = false;
				break;
			case 's':
				/* specify the fps */
				if((fps = atoi(optarg)) <= 0)
				{
					throw "Can't read fps!";
				}
				break;
			case 'p':
				/* start paused */
				game_paused = true;
				break;
			case 'l':
				/* specify the sound-volume */
				music.set_volume(atoi(optarg));
				break;
			case 'd':
				/* disable online-highscore */
				online_highscore = false;
				break;
			case 'e':
				/* enable online-highscore */
				online_highscore = true;
				break;
			case 'n':
				/* specify a name for the online-highscore */
				strncpy(name, optarg, 128);
				online_highscore = true;
				break;
			case 't':
				/* change data prefix path */
				strncpy(data_prefix, optarg, 256);
				break;
			default:
				/* unknown option */
				throw "Unkonwn command-line option!";
				break;
		}
	}
	
	/* initialize the rand() function */
	srand(time(NULL));

	/* initialize the SDL */
	if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0)
	{
		cout << SDL_GetError() << endl;
		throw "SDL_Init failed!";
	}

	/* initialize the TTF-Library */
	if(TTF_Init() < 0)
	{
		cout << TTF_GetError() << endl;
		throw "TTF_Init failed!";
	}

	/* initialize the NET-Library */
	if(SDLNet_Init()< 0)
	{
		cout << SDLNet_GetError() << endl;
		throw "SDLNet_Init failed!";
	}


	/* initialize the sound-mixer */
	if(Mix_OpenAudio(44100, AUDIO_S16SYS, 2, 2048) < 0)
	{
		cout << "Warning: Couldn't open audio stream!" << endl;
		cout << Mix_GetError() << endl;
	}

	/* set SDL_Quit to be called at exit */
	atexit(SDL_Quit);
	set_terminate(SDL_Quit);

	/* initialize the screen */
	init_screen();

	/* make cursor invisible */
	SDL_ShowCursor(SDL_DISABLE);
	SDL_EnableKeyRepeat(1, 100);
	/* initialize the "loading" string */
	standard_big.init("data/fonts/standard.ttf", 40);
	loading.init(standard_big);

	/* update "loading" string */
	loading.render_text("Loading     ", 24, 0, 205);
	loading.draw_centered();
	SDL_Flip(screen);
	SDL_Delay(250);

	/* load the images */
	background.load("data/images/background.jpg");
	logo.load("data/images/logo.bmp", 0, 255, 0);
	logo2.load("data/images/logo2.bmp", 0, 255, 0);

	/* update "Loading" string */
	loading.render_text("Loading.    ", 24, 0, 205);
	loading.draw_centered();
	SDL_Flip(screen);
	SDL_Delay(250);

	/* load the objects */
	ship.init("data/sprites/ship");
	rocket.init("data/sprites/rocket", -5);
	star_monster1.init("data/sprites/star_monster1", "data/sprites/explosion", 1);
	star_monster2.init("data/sprites/star_monster2", "data/sprites/explosion", 1);
	brain_monster.init("data/sprites/brain_monster", "data/sprites/explosion", 2);

	/* update "Loading" string */
	loading.render_text("Loading..   ", 24, 0, 205);
	loading.draw_centered();
	SDL_Flip(screen);
	SDL_Delay(250);

	/* initialize the fonts */
	dots.init("data/fonts/dots.ttf", 40);
	standard_small.init("data/fonts/standard.ttf", 20);
	digital_big.init("data/fonts/digital.ttf", 40);
	digital_small.init("data/fonts/digital.ttf", 15);
	version_string.init(digital_small);
	fps_string.init(digital_small);
	lives_string.init(dots);
	score_string.init(dots);
	pause_string.init(standard_big);
	game_over_string.init(standard_big);
	game_over_txt.init(standard_small);
	info_txt.init(standard_small);
	info_txt2.init(standard_small);
	name_string.init(standard_small);
	name_input_txt.init(standard_small);
	for(i = 0; i < 6; i++)
	{
		highscore[i].name.init(standard_small);
		highscore[i].score.init(standard_small);
		highscore[i].position.init(standard_small);
	}

	/* update "Loading" string */
	loading.render_text("Loading...  ", 24, 0, 205);
	loading.draw_centered();
	SDL_Flip(screen);
	SDL_Delay(250);

	/* set the position of the images */
	background.setxy(0, 70);
	logo.setxy(0, 0);
	logo2.setxy(0, 450);

	/* set the start position of the ship */
	ship.setxy(10, 70 + 200 - (ship.geth() / 2));

	/* stop blinking */
	ship.stop_anim();

	/* set the position of the strings */
	version_string.setxy(600, 460);
	fps_string.setxy(7, 460);
	score_string.setxy(620, 50);
	lives_string.setxy(10, 50);
	game_over_string.sety(270);
	game_over_txt.sety(320);
	info_txt.sety(390);
	info_txt2.sety(420);
	for(i = 0; i < 5; i++)
	{
		highscore[i].score.setxy(260, i * 20 + 100);
		highscore[i].name.setxy(280, i * 20 + 100);
		highscore[i].position.setxy(215, i * 20 + 100);
	}
	highscore[i].score.setxy(260, 225);
	highscore[i].name.setxy(280, 225);
	highscore[i].position.setxy(215, 225);
	name_string.setxy(195, 245);
	name_input_txt.setxy(190, 210);

	/* render text */
	version_string.render_text(VERSION, 255, 255, 255);
	pause_string.render_text("Game paused!", 24, 0, 205);
	snprintf(tmp, 256, "%d", LIVES);
	lives_string.render_text(tmp, 255, 255, 255);
	score_string.render_text("0", 255, 255, 255);
	game_over_string.render_text("Game over!", 24, 0, 205);
	game_over_txt.render_text("Press return to continue!", 205, 0, 24);
	info_txt.render_text("(c) Dominik Hackl [dominik@hackl.dhs.org]", 24, 0, 205);
	info_txt2.render_text("Homepage: http://www.hackl.dhs.org/spacerider", 24, 0, 205);
	name_input_txt.render_text("Please enter your name:", 255, 255, 255);

	/* set up boxes */
	highscore_box1.set(170, 102, 280, 147, 255, 255, 255, 40);
	highscore_box2.set(170, 102, 280, 102, 255, 255, 255, 30);
	highscore_box3.set(170, 226, 280, 23, 255, 255, 255, 30);
	name_box1.set(180, 200, 250, 80, 255, 255, 255, 40);
	name_box2.set(190, 240, 230, 30, 255, 255, 255, 30);
	cursor.set(196 + 10 * strlen(name), 243, 2, 24, 255, 255, 255, 130);

	/* update "Loading" string */
	loading.render_text("Loading.... ", 24, 0, 205);
	loading.draw_centered();
	SDL_Flip(screen);
	SDL_Delay(250);
	
	music.load("data/sound/moon.xm");
	
	/* update "Loading" string */
	loading.render_text("Loading.....", 24, 0, 205);
	loading.draw_centered();
	SDL_Flip(screen);
	SDL_Delay(250);
}

Spacerider::~Spacerider()
{
	if(screen != NULL)
	{
		/* free screen */
		SDL_FreeSurface(screen);
	}
	/* call the TTF-Library cleanup function */
	TTF_Quit();

	/* call the NET-Library cleanup function */
	SDLNet_Quit();
	
	/* call the SDL cleanup function */
	SDL_Quit();
}

void Spacerider::init_screen()
{
	/* initialize the screen */
	if(fullscreen)
	{
		screen = SDL_SetVideoMode(640, 480, 24, SDL_HWSURFACE | SDL_FULLSCREEN);
		//screen = SDL_SetVideoMode(1024, 768, 24, SDL_HWSURFACE | SDL_FULLSCREEN);
	}
	else
	{
		screen = SDL_SetVideoMode(640, 480, 24, SDL_HWSURFACE);
	}

	if(screen == NULL)
	{
		cout << "Unable to set 640x480 video: " <<  SDL_GetError() << endl;
		throw "SDL_SetVideoMode failed!";
	}
}

void Spacerider::parse_events()
{
	SDL_Event event;
	int n;
	Uint8 *keys;

	/* enable unicode for input-mode */
	if(name_input)
	{
		SDL_EnableUNICODE(1);
	}
	else
	{
		SDL_EnableUNICODE(0);
	}
	
	/* get an event */
	while(SDL_PollEvent(&event))
	{
		if(event.type == SDL_QUIT)
		{
			/* quit */
			throw "";
		}
		else if(event.type == SDL_KEYDOWN)
		{
			if(event.key.keysym.sym == SDLK_ESCAPE)
			{
				/* escape was pressed, quit! */
				throw "";
			}
			
			/* we are in input-mode */
			if(name_input)
			{
				if((event.key.keysym.sym >= '0' && event.key.keysym.sym <= '9') || (event.key.keysym.sym >= 'a' && event.key.keysym.sym <= 'z') || (event.key.keysym.sym >= 'A' && event.key.keysym.sym <= 'Z') || event.key.keysym.sym == ' ')
				{
					if((n = strlen(name)) < 15)
					{
						name[n] = event.key.keysym.unicode;
						name[n + 1] = '\0';
					}
				}
				else if(event.key.keysym.sym == SDLK_BACKSPACE)
				{
					name[strlen(name) - 1] = '\0';
				}
				else if(event.key.keysym.sym == SDLK_RETURN)
				{
					SDL_EnableKeyRepeat(0, 0);
					name_input = false;
				}
				else if(event.key.keysym.sym == SDLK_END)
				{
					name[0] = '\0';
				}
				cursor.set(196 + 10 * strlen(name), 243, 2, 24, 255, 255, 255, 100);
				continue;
			}
			
			/* we are in normal mode */
			if(event.key.keysym.sym == SDLK_q)
			{
				/* q was pressed, quit! */
				throw "";
			}
			else if(event.key.keysym.sym == SDLK_RETURN)
			{
				/* continue game, after game over! */
				if(game_over)
				{
					if(online_highscore && !score_sent && highscore_thread_started)
					{
						SDL_KillThread(highscore_thread);
					}
					lives = LIVES;
					score = 0;
					ship.make_visible();
					ship.stop_anim();
					ship.set_frame(0);
					brain_monster.clear();
					star_monster1.clear();
					star_monster2.clear();
					ship.setxy(10, 70 + 200 - (ship.geth() / 2));
					game_over = false;
					score_sent = false;
					highscore_thread_started = false;
				}

			}
			else if(event.key.keysym.sym == SDLK_SPACE)
			{
				if(!game_paused && !game_over)
				{
					/* we are not in game_paused, so we shoot */
					shoot();
				}
				else if(game_paused)
				{
					/* cancel game_paused */
					game_paused = false;
				}
			}
			else if(event.key.keysym.sym == SDLK_p)
			{
				/* invert game_paused */
				if(!game_over)
				{
					game_paused = !game_paused;
				}
			}
			else if(event.key.keysym.sym == SDLK_f)
			{
				/* invert fullscreen */
				fullscreen = !fullscreen;

				/* reinit screen */
				init_screen();
			}
			else if(event.key.keysym.sym == SDLK_r)
			{
				/* reset game */
				if(online_highscore && !score_sent && highscore_thread_started)
				{
					SDL_KillThread(highscore_thread);
				}
				lives = LIVES;
				score = 0;
				ship.make_visible();
				ship.stop_anim();
				ship.set_frame(0);
				brain_monster.clear();
				star_monster1.clear();
				star_monster2.clear();
				rocket.clear();
				ship.setxy(10, 70 + 200 - (ship.geth() / 2));
				score_sent = false;
				game_over = false;
				name_input = true;
				highscore_thread_started = false;
				SDL_EnableKeyRepeat(1, 100);
			}
		}
	}

	/* get keys snapshot */
	keys = SDL_GetKeyState(NULL);

	if(!name_input && !game_paused && !game_over)
	{
		if(keys[SDLK_UP])
		{
			/* move ship up */
			ship.addy((int)(-2 * dt));
			if(ship.gety() < 70)
			{
				ship.sety(70);
			}
		}
		if(keys[SDLK_DOWN])
		{
			/* move ship down */
			ship.addy((int)(2 * dt));
			if(ship.gety() > 470 - ship.geth())
			{
				ship.sety(470 - ship.geth());
			}
		}
		if(keys[SDLK_LEFT])
		{
			/* move ship left */
			ship.addx((int)(-2 * dt));
			if(ship.getx() < 0)
			{
				ship.setx(0);
			}
		}
		if(keys[SDLK_RIGHT])
		{
			/* move ship right */
			ship.addx((int)(2 * dt));
			if(ship.getx() > 640 - ship.getw())
			{
				ship.setx(640 - ship.getw());
			}
		}
	}
	if(keys[SDLK_COMMA] || keys[SDLK_KP_MINUS])
	{
		/* decrease music volume */
		music.set_volume(music.get_volume() - 1);
	}
	if(keys[SDLK_PERIOD] || keys[SDLK_KP_PLUS])
	{
		/* increase music volume */
		music.set_volume(music.get_volume() + 1);
	}
	if(keys[SDLK_PAGEUP])
	{
		/* increase fps */
		fps++;
	}
	if(keys[SDLK_PAGEDOWN])
	{
		/* prevent devision through zero */
		if(fps > 1)
		{
			/* decrease fps */
			fps--;
		}
	}
}

void Spacerider::loop()
{
	int n = 100, pscore = 0, plives = 0;
	Uint32 td = 0, td2, tn = SDL_GetTicks(), next_frame = 0;
	char tmp[256];

	/* game loop */
	while(1)
	{
		/* frame calculations */
		td2 = SDL_GetTicks();
		dt = ((float)(td2 - td)) * 0.1;
		td = td2;
		
		/* let the music play */
		music.play();
		
		if(n >= 100)
		{
			/* calculate the time needed for the last 100 frames */
			tn = SDL_GetTicks() - tn;

			/* avoid a devision through zero */
			if(tn > 0)
			{
				/* display the frames per second */
				snprintf(tmp, 256, "%d fps", 100000 / tn);
			}
			else
			{
				/* no statistic now */
				strncpy(tmp, "-- fps", 256);
			}
			
			/* render text */
			fps_string.render_text(tmp, 255, 255, 255);

			/* reset everything */
			tn = SDL_GetTicks();
			n = 0;
		}
		n++;
		if(game_over && online_highscore && !score_sent && !highscore_thread_started && !rocket.alive())
		{
			/* send score to the highscore-server */
			highscore_thread = SDL_CreateThread(&send_score_thread, this);
			highscore_thread_started = true;
		}
		
		/* display lives */
		if(lives != plives)
		{
			snprintf(tmp, 256, "%d", lives);
			lives_string.render_text(tmp, 255, 255, 255);
			plives = lives;
		}
		
		/* display score */
		if(score != pscore)
		{
			snprintf(tmp, 256, "%d", score);
			score_string.render_text(tmp, 255, 255, 255);
			pscore = score;
		}

		/* parse events */
		parse_events();
		
		if(!game_paused && !name_input)
		{
			gt += dt * 10;
		
			/* check for collisions between the bullets and the enemies, and add point(s) to the score */
			score += rocket.collision(brain_monster) * 3;
			score += rocket.collision(star_monster1);
			score += rocket.collision(star_monster2);
		
			/* move the bullets */
			rocket.move();
			
			/* move the enemies */
			brain_monster.move();
			star_monster1.move();
			star_monster2.move();

			/* if we are not in god-mode and we hit one of the enymies */
			if(ship.is_visible() && !ship.is_animating() && (brain_monster.hard_collision(ship) || star_monster1.hard_collision(ship) || star_monster2.hard_collision(ship)))
			{
				/* we lose one life */
				lives--;

				/* are we dead ??? */
				if(lives <= 0)
				{
					/* yes we are :( */
					game_over = true;
					ship.make_invisible();

				}
				else
				{
					/* set the ship back to the start-position */
					ship.setxy(10, 70 + 200 - (ship.geth() / 2));

					/* start blinking */
					ship.start_anim();
					dtime = (Uint32)gt;
				}
			}

			/* if we are in god-mode and the god-mode time is over, we go to normal mode again */
			if(ship.is_animating() && gt - dtime > 3000)
			{
				/* start blinking */
				ship.stop_anim();

				/* go back to the first frame */
				ship.set_frame(0);
			}

			/* the more points we have, the more enemies will join the game */
			if(gt - last_enemy > 750 / (((score + 1) / DIFFICULTNESS) + 1))
			{
				/* add one random enemy */
				switch(rand() % 3)
				{
					case 0:
					add_enemy(brain_monster);
					break;
					case 1:
					add_enemy(star_monster1);
					break;
					case 2:
					add_enemy(star_monster2);
					break;
				}
				/* reset counter */
				last_enemy = (Uint32)gt;
			}

		}

		if(!game_paused)
		{
			/* scroll the background image */
			scroll += dt;
			
			/* if scroll is greater than the length of the background image */
			if(scroll >= 3909)
			{
				/* reset scroll */
				scroll -= 3909;
			}
		}

		if(name_input)
		{
			name_string.render_text(name, 255, 255, 255);
		}
		
		/* draw the frame */
		draw_frame();

		/* frame calcuations */
		next_frame += 1000 / fps;
		while(SDL_GetTicks() < next_frame)
		{
			SDL_Delay(1);
		}
	}

}

void Spacerider::draw_frame()
{
	/* clear screen */
	SDL_FillRect(screen, 0, 0);

	/* draw background image */
	draw_scroll();

	/* draw bullets */
	rocket.draw();

	/* draw enemies */
	brain_monster.draw();
	star_monster1.draw();
	star_monster2.draw();
	
	/* draw ship */
	if(!name_input)
	{
		ship.draw();
	}

	/* draw logo */
	logo.draw();
	logo2.draw();

	/* draw strings */
	version_string.draw();
	fps_string.draw();
	lives_string.draw();
	score_string.draw_justified();
	
	if(game_over)
	{
		game_over_string.draw_hcentered();
		game_over_txt.draw_hcentered();
		info_txt.draw_hcentered();
		info_txt2.draw_hcentered();
		if(online_highscore && score_sent)
		{
			draw_highscore();
		}
	}
	else if(game_paused)
	{
		pause_string.draw_centered();
	}

	if(name_input)
	{
		name_box1.draw();
		name_box2.draw();
		name_string.draw();
		name_input_txt.draw();
		cursor.draw();
	}

	/* refresh screen */
	SDL_Flip(screen);
}

void Spacerider::add_enemy(Enemy &a)
{
	a.add(70 + (ship.geth() / 2) - (a.geth() / 2) + rand() % (400 - a.geth() - 2 * ((ship.geth() / 2) - (a.geth() / 2))), (rand() % 3) + 1);
}

void Spacerider::shoot()
{
	rocket.add(ship, 20, 0);
}

void Spacerider::draw_scroll()
{
	/* do we have to split up the background image */
	if(scroll > 3909 - 640)
	{
		/* draw the rest of the image */
		background.draw_pice((int)(3909 - scroll), 400, (int)scroll, 0);

		/* draw the rest */
		background.setx((int)(3909 - scroll - 1));
		background.draw_pice((int)(640 - (3909 - scroll)), 400, 0, 0);
		background.setx(0);
	}
	else
	{
		/* draw the background image */
		background.draw_pice(640, 400, (int)scroll, 0);
	}
}

void Spacerider::draw_highscore()
{
	int i;
	
	/* draw background-boxes */
	highscore_box1.draw();
	highscore_box2.draw();
	highscore_box3.draw();
		

	/* draw highscore */
	for(i = 0; i < 6; i++)
	{
		highscore[i].position.draw_justified();
		highscore[i].score.draw_justified();
		highscore[i].name.draw();
	}
}

int send_score_thread(void *param)
{
	IPaddress ip;
	TCPsocket sock;
	char str[1024], tmp[128], tmp2[128];
	int i, n = 0, a;
	Spacerider *b = (Spacerider *)param;
	Spacerider &e = *b;

	/* check if a name is set */
	if(e.name[0] == '\0')
	{
		return 0;
	}

	/* try to resolve address */
	if(SDLNet_ResolveHost(&ip, "www.hackl.dhs.org", 80) == -1)
	{
		return 0;
	}

	/* open connection */
	if(!(sock = SDLNet_TCP_Open(&ip)))
	{
		return 0;
	}
	
	for(i = 0, n = 0; e.name[i] != '\0'; i++, n++)
	{
		if(e.name[i] == ' ')
		{
			tmp[n++] = '%';
			tmp[n++] = '2';
			tmp[n] = '0';
		}
		else
		{
			tmp[n] = e.name[i];
		}
	}
	tmp[n] = '\0';
	n = 0;

	/* create HTTP-GET-request */
	/* PLEASE DON'T CHANGE ANYTHING HERE, CHEATING IS NOT FAIR! */
	/* YOUR IP-ADDRESS WILL GET BLOGGED IF YOU TRY TO CHEAT!!! */
	snprintf(str, 1024, "GET http://www.hackl.dhs.org/spacerider/set_highscore.php?version=%s&os=%s&lives=%d&name=%s&score=%d\n", VERSION, OS, LIVES, tmp, e.score);


	/* send request */
	SDLNet_TCP_Send(sock, str, strlen(str) + 1);

	while(n < 6)
	{
		/* fill string with '\0's */
		memset(str, '\0', 1024);

		/* receive highscore-list */
		SDLNet_TCP_Recv(sock, str, 1024);

		for(i = 0; n < 6 && i < 1024 && str[i] != '\0'; i++)
		{
			if(n >= 5)
			{
				sscanf(str + i, "%d\n", &a);
				str[i + 3] = '\0';
				if(a > 0 && a < 6)
				{
					/* we are in the top 5 */
					a--;
					e.highscore[a].name.render_text(e.name, 255 - a * 20, 0, 0);
					snprintf(tmp, 128, "%d", e.score);
					e.highscore[a].score.render_text(tmp, 255 - a * 20, 0, 0);
					e.highscore[n].position.make_invisible();
					e.highscore[n].name.make_invisible();
					e.highscore[n].score.make_invisible();
					e.highscore_box3.make_invisible();
					e.highscore_box1.make_invisible();
					e.highscore_box2.set_color(255, 255, 255, 70);
				}
				else
				{
					/* we are somewhere behind the top 5 */
					if(a > 999)
					{
						snprintf(tmp, 128, "--");
					}
					else
					{
						snprintf(tmp, 128, "%d.", a);
					}
					e.highscore[n].position.render_text(tmp, 255 - n * 20, 0, 0);
					snprintf(tmp, 128, "%d", e.score);
					e.highscore[n].score.render_text(tmp, 255 - n * 20, 0, 0);
					e.highscore[n].name.render_text(e.name, 255 - n * 20, 0, 0);
					e.highscore[n].position.make_visible();
					e.highscore[n].name.make_visible();
					e.highscore[n].score.make_visible();
					e.highscore_box3.make_visible();
					e.highscore_box1.make_visible();
					e.highscore_box2.set_color(255, 255, 255, 30);
				}
			}
			else
			{
				/* read highscore, and save it in the highscore struct */
				sscanf(str + i, "%s - %s\n", tmp, tmp2);
				
				tmp2[17] = '\0';
				tmp[4] = '\0';
				
				e.highscore[n].score.render_text(tmp, 255, 255, 255);
				e.highscore[n].name.render_text(tmp2, 255, 255, 255);
				
				snprintf(tmp, 128, "%d.", n + 1);
				{
					e.highscore[n].position.render_text(tmp, 255 - n * 20, 0, 0);
				}
			
				/* go to the end of the line */
				while(i < 1024 && str[i] != '\0' && str[i] != '\n')
				{
					i++;
				}
				
			}
			n++;
		}
	}

	/* close connection */
	SDLNet_TCP_Close(sock);
	e.score_sent = true;
	return 1;
}

