#include "sprite.h"
#include "features.h"
#include "globals.h"

#include <iostream>
#include <string>
#include <fstream>

using namespace std;

BSprite::BSprite() : init_done(false)
{
}

BSprite::BSprite(const char *path) : init_done(false)
{
	init(path);
}

BSprite::~BSprite()
{
	int i;
	if(is_init_done())
	{
		for(i = 0; check_frame(i); i++)
		{
			free_img(get_image(i));
		}
		delete frame;
	}
}

void BSprite::init(const char *path)
{
	char filename[256], buffer[256], name[256];
	ifstream file;
	int n, r, g, b;

	if(is_init_done())
	{
		cout << "init already done!" << endl;
		throw "BSprite::init failed!";
	}

	snprintf(filename, 256, "%s%s/info", data_prefix, path);

	file.open(filename, ios_base::in);
	if(!file)
	{
		cout << "Can't open sprite info file: \"" << filename << "\"." << endl;
		throw "BSprite::init failed!";
	}
	
	try
	{
		init_done = true;
		
		file.getline(buffer, 256);
		sscanf(buffer, "FRAMES: %u", &nr);

		frame = new frame_struct[get_frames()];

		for(n = 0; !file.eof() && check_frame(n); n++)
		{
			file.getline(buffer, 256);
			if(buffer[0] != '\r' && buffer[0] != '\0' && buffer[0] != '\n')
			{
				sscanf(buffer, "%s %d %d %d %d", name, &frame[n].pause, &r, &g, &b);
				snprintf(filename, 256, "%s/%s", path, name);

				if(r >= 0)
				{
					frame[n].image = load_img(filename, r, g, b);
				}
				else
				{
					frame[n].image = load_img(filename);
				}
			} 
		}
	}
	catch(...)
	{
		file.close();
		throw;
	}
	file.close();
}

void BSprite::reset()
{
	int i;
	if(!is_init_done())
	{
		cout << "init not done!" << endl;
		throw "BSprite::reset failed!";
	}
	init_done = false;
	
	for(i = 0; check_frame(i); i++)
	{
		free_img(get_image(i));
	}
	delete []frame;
}

int BSprite::geth()
{
	if(is_init_done())
	{
		return get_image(0)->h;
	}
	return 0;
}

int BSprite::getw()
{
	if(is_init_done())
	{
		return get_image(0)->w;
	}
	return 0;
}

int BSprite::get_frames()
{
	if(is_init_done())
	{
		return nr;
	}
	return 0;
}

bool BSprite::is_init_done()
{
	return init_done;
}

int BSprite::get_pause(const int f)
{
	if(check_frame(f))
	{
		return frame[f].pause;
	}
	return 0;
}

SDL_Surface *BSprite::get_image(const int f)
{
	if(check_frame(f))
	{
		return frame[f].image;
	}
	return NULL;
}

bool BSprite::check_frame(const int i)
{
	if(is_init_done() && i < get_frames() && i >= 0)
	{
		return true;
	}
	return false;
}

Sprite::Sprite() : last_update(0), our_base(false), base(NULL)
{
	first_init();
}

Sprite::Sprite(const char *path) : last_update(0), our_base(false), base(NULL)
{
	first_init();
	init(path);
}

Sprite::Sprite(BSprite &b) : last_update(0), our_base(false), base(NULL)
{
	first_init();
	init(b);
}

Sprite::~Sprite()
{
	if(our_base)
	{
		delete base;
	}
}

void Sprite::first_init()
{
	setxy(0, 0);
	set_speed(1);
	start_anim();
	set_frame(0);
	enable_loop();
}

void Sprite::init(const char *path)
{
	if(base != NULL)
	{
		printf("init already done!\n");
		throw "Sprite::init failed!";
	}
	
	try
	{
		base = new BSprite(path);
		if(base->get_frames() > 1)
		{
			start_anim();
		}
	}
	catch(...)
	{
		delete base;
		base = NULL;
		throw;
	}
	our_base = true;
}

void Sprite::init(BSprite &b)
{
	if(base != NULL)
	{
		printf("init already done!\n");
		throw "Sprite::init failed!";
	}

	if(!b.is_init_done())
	{
		printf("init of base not done!\n");
		throw "Sprite::init failed!";
	}
	
	base = &b;
	
	if(base->get_frames() > 1)
	{
		start_anim();
	}
}

void Sprite::enable_loop()
{
	play_loop = true;
}

void Sprite::disable_loop()
{
	play_loop = false;
}

void Sprite::start_anim()
{
	animating = true;
}

void Sprite::stop_anim()
{
	animating = false;
}

bool Sprite::is_animating()
{
	return animating;
}

void Sprite::set_frame(const int n)
{
	if(base == NULL)
	{
		curr = 0;
		return;
	}
	curr = n % base->get_frames();
}

int Sprite::get_frames()
{
	if(base == NULL)
	{
		return 0;
	}
	return base->get_frames();
}

void Sprite::draw()
{
	if(base == NULL)
	{
		return;
	}

	if(!is_visible())
	{
		return;
	}

	if(is_animating() && last_update + base->get_pause(curr) * speed < SDL_GetTicks())
	{
		if(!play_loop && curr + 1 >= base->get_frames())
		{
			stop_anim();
			make_invisible();
			return;
		}
		set_frame(curr + 1);
		last_update = SDL_GetTicks();
	}
	draw_img(base->get_image(curr), (int)getx(), (int)gety());
}

float Sprite::get_speed()
{
	return speed;
}

void Sprite::set_speed(float s)
{
	if(s > 0)
	{
		speed = s;
	}
}

int Sprite::getw()
{
	if(base == NULL)
	{
		return 0;
	}
	return base->getw();
}

int Sprite::geth()
{
	if(base == NULL)
	{
		return 0;
	}
	return base->geth();
}


